# -*- coding: utf-8 -*-
#
# This file is part of the GofGateway project
#
# Elettra-Sincrotrone Trieste S.C.p.A. di interesse nazionale
# Strada Statale 14 - km 163,5 in AREA Science Park
# 34149 Basovizza, Trieste ITALY
#
# Distributed under the terms of the GPL license.
# See LICENSE.txt for more info.

""" gof-gateway

simple server to poll and expose via events a number of gof attributes.
The events will be used by tha alaram system (maybe also hdb++)
"""

# PyTango imports
import PyTango
from PyTango import DebugIt
from PyTango.server import run
from PyTango.server import Device, DeviceMeta
from PyTango.server import attribute, command
from PyTango.server import device_property
from PyTango import AttrQuality, DispLevel, DevState
from PyTango import AttrWriteType, PipeWriteType
# Additional import
# PROTECTED REGION ID(GofGateway.additionnal_import) ENABLED START #
# PROTECTED REGION END #    //  GofGateway.additionnal_import

__all__ = ["GofGateway", "main"]


class GofGateway(Device):
    """
    simple server to poll and expose via events a number of gof attributes.
    The events will be used by tha alaram system (maybe also hdb++)
    """
    __metaclass__ = DeviceMeta
    # PROTECTED REGION ID(GofGateway.class_variable) ENABLED START #
    # PROTECTED REGION END #    //  GofGateway.class_variable

    # -----------------
    # Device Properties
    # -----------------

    Facility = device_property(
        dtype='str',
        mandatory=True
    )

    # ----------
    # Attributes
    # ----------

    MCPLLStatus_S1_7 = attribute(
        dtype='bool',
        label="BPM S1.7/MCPPLStatus",
    )

    MCPLLStatus_S7_7 = attribute(
        dtype='bool',
        label="BPM S5.7/MCPPLStatus",
    )

    MCPLLStatus_S9_7 = attribute(
        dtype='bool',
        label="BPM S9.7/MCPPLStatus",
    )

    WaitTimeCurrent01 = attribute(
        dtype='int',
        label="ecgof1 waitTimeCurrent",
    )

    WaitTimeCurrent02 = attribute(
        dtype='int',
        label="ecgof2 waitTimeCurrent",
    )

    WaitTimeCurrent03 = attribute(
        dtype='int',
        label="ecgof2 waitTimeCurrent",
    )

    WaitTimeCurrent04 = attribute(
        dtype='int',
        label="ecgof4 waitTimeCurrent",
    )

    WaitTimeCurrent05 = attribute(
        dtype='int',
        label="ecgof5 waitTimeCurrent",
    )

    WaitTimeCurrent06 = attribute(
        dtype='int',
        label="ecgof5 waitTimeCurrent",
    )

    WaitTimeCurrent07 = attribute(
        dtype='int',
        label="ecgof7 waitTimeCurrent",
    )

    WaitTimeCurrent08 = attribute(
        dtype='int',
        label="ecgof8 waitTimeCurrent",
    )

    WaitTimeCurrent09 = attribute(
        dtype='int',
        label="ecgof9 waitTimeCurrent",
    )

    WaitTimeCurrent10 = attribute(
        dtype='int',
        label="ecgof10 waitTimeCurrent",
    )

    WaitTimeCurrent11 = attribute(
        dtype='int',
        label="ecgof11 waitTimeCurrent",
    )

    WaitTimeCurrent12 = attribute(
        dtype='int',
        label="ecgof12 waitTimeCurrent",
    )

    # ---------------
    # General methods
    # ---------------

    def init_device(self):
        Device.init_device(self)
        # PROTECTED REGION ID(GofGateway.init_device) ENABLED START #
        # create attribute proxies for all the attributes ttha must be read in and replicated
        self.set_state(PyTango.DevState.INIT)
        self.debug_stream('using facility '+self.Facility)
        try:
            self.wtc01 = PyTango.AttributeProxy(self.Facility + 'ecgofa1/mv64360mon/0/WaitTimeCurrent')
            self.wtc02 = PyTango.AttributeProxy(self.Facility + 'ecgofa2/mv64360mon/0/WaitTimeCurrent')
            self.wtc03 = PyTango.AttributeProxy(self.Facility + 'ecgofa3/mv64360mon/0/WaitTimeCurrent')
            self.wtc04 = PyTango.AttributeProxy(self.Facility + 'ecgofa4/mv64360mon/0/WaitTimeCurrent')
            self.wtc05 = PyTango.AttributeProxy(self.Facility + 'ecgofa5/mv64360mon/0/WaitTimeCurrent')
            self.wtc06 = PyTango.AttributeProxy(self.Facility + 'ecgofa6/mv64360mon/0/WaitTimeCurrent')
            self.wtc07 = PyTango.AttributeProxy(self.Facility + 'ecgofa7/mv64360mon/0/WaitTimeCurrent')
            self.wtc08 = PyTango.AttributeProxy(self.Facility + 'ecgofa8/mv64360mon/0/WaitTimeCurrent')
            self.wtc09 = PyTango.AttributeProxy(self.Facility + 'ecgofa9/mv64360mon/0/WaitTimeCurrent')
            self.wtc10 = PyTango.AttributeProxy(self.Facility + 'ecgofa10/mv64360mon/0/WaitTimeCurrent')
            self.wtc11 = PyTango.AttributeProxy(self.Facility + 'ecgofa11/mv64360mon/0/WaitTimeCurrent')
            self.wtc12 = PyTango.AttributeProxy(self.Facility + 'ecgofa12/mv64360mon/0/WaitTimeCurrent')

            self.bpm17 = PyTango.AttributeProxy(self.Facility + 'sr/DIAGNOSTICS/BPM_S1.7/MCPLLStatus')
            self.bpm57 = PyTango.AttributeProxy(self.Facility + 'sr/DIAGNOSTICS/BPM_S5.7/MCPLLStatus')
            self.bpm97 = PyTango.AttributeProxy(self.Facility + 'sr/DIAGNOSTICS/BPM_S9.7/MCPLLStatus')

            self.set_state(PyTango.DevState.ON)
            self.set_status('connected to attributes')

        except PyTango.DevFailed as e:
            self.error_stream(str(e))
            self.fatal_stream(str(e))
            self.set_state(PyTango.DevState.UNKNOWN)
            self.set_status('failed to connetct to some attribute - cannot continue')


        # PROTECTED REGION END #    //  GofGateway.init_device

    def always_executed_hook(self):
        # PROTECTED REGION ID(GofGateway.always_executed_hook) ENABLED START #
        pass
        # PROTECTED REGION END #    //  GofGateway.always_executed_hook

    def delete_device(self):
        # PROTECTED REGION ID(GofGateway.delete_device) ENABLED START #
        pass
        # PROTECTED REGION END #    //  GofGateway.delete_device

    # ------------------
    # Attributes methods
    # ------------------

    def read_MCPLLStatus_S1_7(self):
        # PROTECTED REGION ID(GofGateway.MCPLLStatus_S1_7_read) ENABLED START #
        return self.bpm17.read().value
        # PROTECTED REGION END #    //  GofGateway.MCPLLStatus_S1_7_read

    def is_MCPLLStatus_S1_7_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_MCPLLStatus_S1_7_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_MCPLLStatus_S1_7_allowed

    def read_MCPLLStatus_S7_7(self):
        # PROTECTED REGION ID(GofGateway.MCPLLStatus_S7_7_read) ENABLED START #
        return self.bpm57.read().value
        # PROTECTED REGION END #    //  GofGateway.MCPLLStatus_S7_7_read

    def is_MCPLLStatus_S7_7_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_MCPLLStatus_S7_7_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_MCPLLStatus_S7_7_allowed

    def read_MCPLLStatus_S9_7(self):
        # PROTECTED REGION ID(GofGateway.MCPLLStatus_S9_7_read) ENABLED START #
        return self.bpm97.read().value
        # PROTECTED REGION END #    //  GofGateway.MCPLLStatus_S9_7_read

    def is_MCPLLStatus_S9_7_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_MCPLLStatus_S9_7_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_MCPLLStatus_S9_7_allowed

    def read_WaitTimeCurrent01(self):
        # PROTECTED REGION ID(GofGateway.WaitTimeCurrent01_read) ENABLED START #
        return self.wtc01.read().value
        # PROTECTED REGION END #    //  GofGateway.WaitTimeCurrent01_read

    def is_WaitTimeCurrent01_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_WaitTimeCurrent01_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_WaitTimeCurrent01_allowed

    def read_WaitTimeCurrent02(self):
        # PROTECTED REGION ID(GofGateway.WaitTimeCurrent02_read) ENABLED START #
        return self.wtc02.read().value
        # PROTECTED REGION END #    //  GofGateway.WaitTimeCurrent02_read

    def is_WaitTimeCurrent02_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_WaitTimeCurrent02_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_WaitTimeCurrent02_allowed

    def read_WaitTimeCurrent03(self):
        # PROTECTED REGION ID(GofGateway.WaitTimeCurrent03_read) ENABLED START #
        return self.wtc03.read().value
        # PROTECTED REGION END #    //  GofGateway.WaitTimeCurrent03_read

    def is_WaitTimeCurrent03_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_WaitTimeCurrent03_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_WaitTimeCurrent03_allowed

    def read_WaitTimeCurrent04(self):
        # PROTECTED REGION ID(GofGateway.WaitTimeCurrent04_read) ENABLED START #
        return self.wtc04.read().value
        # PROTECTED REGION END #    //  GofGateway.WaitTimeCurrent04_read

    def is_WaitTimeCurrent04_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_WaitTimeCurrent04_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_WaitTimeCurrent04_allowed

    def read_WaitTimeCurrent05(self):
        # PROTECTED REGION ID(GofGateway.WaitTimeCurrent05_read) ENABLED START #
        return self.wtc05.read().value
        # PROTECTED REGION END #    //  GofGateway.WaitTimeCurrent05_read

    def is_WaitTimeCurrent05_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_WaitTimeCurrent05_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_WaitTimeCurrent05_allowed

    def read_WaitTimeCurrent06(self):
        # PROTECTED REGION ID(GofGateway.WaitTimeCurrent06_read) ENABLED START #
        return self.wtc06.read().value
        # PROTECTED REGION END #    //  GofGateway.WaitTimeCurrent06_read

    def is_WaitTimeCurrent06_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_WaitTimeCurrent06_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_WaitTimeCurrent06_allowed

    def read_WaitTimeCurrent07(self):
        # PROTECTED REGION ID(GofGateway.WaitTimeCurrent07_read) ENABLED START #
        return self.wtc07.read().value
        # PROTECTED REGION END #    //  GofGateway.WaitTimeCurrent07_read

    def is_WaitTimeCurrent07_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_WaitTimeCurrent07_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_WaitTimeCurrent07_allowed

    def read_WaitTimeCurrent08(self):
        # PROTECTED REGION ID(GofGateway.WaitTimeCurrent08_read) ENABLED START #
        return self.wtc08.read().value
        # PROTECTED REGION END #    //  GofGateway.WaitTimeCurrent08_read

    def is_WaitTimeCurrent08_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_WaitTimeCurrent08_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_WaitTimeCurrent08_allowed

    def read_WaitTimeCurrent09(self):
        # PROTECTED REGION ID(GofGateway.WaitTimeCurrent09_read) ENABLED START #
        return self.wtc09.read().value
        # PROTECTED REGION END #    //  GofGateway.WaitTimeCurrent09_read

    def is_WaitTimeCurrent09_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_WaitTimeCurrent09_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_WaitTimeCurrent09_allowed

    def read_WaitTimeCurrent10(self):
        # PROTECTED REGION ID(GofGateway.WaitTimeCurrent10_read) ENABLED START #
        return self.wtc10.read().value
        # PROTECTED REGION END #    //  GofGateway.WaitTimeCurrent10_read

    def is_WaitTimeCurrent10_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_WaitTimeCurrent10_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_WaitTimeCurrent10_allowed

    def read_WaitTimeCurrent11(self):
        # PROTECTED REGION ID(GofGateway.WaitTimeCurrent11_read) ENABLED START #
        return self.wtc11.read().value
        # PROTECTED REGION END #    //  GofGateway.WaitTimeCurrent11_read

    def is_WaitTimeCurrent11_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_WaitTimeCurrent11_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_WaitTimeCurrent11_allowed

    def read_WaitTimeCurrent12(self):
        # PROTECTED REGION ID(GofGateway.WaitTimeCurrent12_read) ENABLED START #
        return self.wtc12.read().value
        # PROTECTED REGION END #    //  GofGateway.WaitTimeCurrent12_read

    def is_WaitTimeCurrent12_allowed(self, attr):
        # PROTECTED REGION ID(GofGateway.is_WaitTimeCurrent12_allowed) ENABLED START #
        return self.get_state() not in [DevState.UNKNOWN,DevState.INIT]
        # PROTECTED REGION END #    //  GofGateway.is_WaitTimeCurrent12_allowed


    # --------
    # Commands
    # --------

# ----------
# Run server
# ----------


def main(args=None, **kwargs):
    # PROTECTED REGION ID(GofGateway.main) ENABLED START #
    return run((GofGateway,), args=args, **kwargs)
    # PROTECTED REGION END #    //  GofGateway.main

if __name__ == '__main__':
    main()
